/*******************************************************************************
* FILE NAME: ifi_utilities.h
*
* DESCRIPTION: 
*  This is the include file which corresponds to ifi_utilities.c
*  It contains some aliases and function prototypes used in that file.
*
* USAGE:
*  This file should not be modified by the user.
*  DO NOT EDIT THIS FILE!
*******************************************************************************/

#ifndef __ifi_utilities_h_
#define __ifi_utilities_h_

#ifndef __general_h__
#include "general.h"
#endif

/* 
 *  Dividers for oscillator driving usart.  One of these should always
 *  be the last argument to usart_open().  These constants are for Vex.
 *  Other PIC devices may use a different oscillator speed.
 */

typedef enum
{
    BAUD_300 = 8064,
    BAUD_600 = 4032,
    BAUD_1200 = 2016,
    BAUD_2400 = 1008,
    BAUD_4800 = 504,
    BAUD_9600 = 252,
    BAUD_19200 = 126,
    BAUD_38400 = 63,
    BAUD_57600 = 42,
    BAUD_115200 = 21
}   usart_speed_t;


/**
 * \addtogroup usart
 *  @{
 */

/**
 *  Returns TRUE if the usart is ready to send another character
 *  or a new character has been received.
 */

/*
 *  History:
 *      Dec 2008    J Bacon     Pulled from Vex default code.
 */

#ifndef _SIMULATOR
#define USART_READY PIR1bits.TXIF
#else
#define USART_READY TRUE
#endif

/** @} */

/* vex_usart.c */
char usart_ready(void);
void usart_init(void);
void _user_putc(char ch);

#endif

